#!/usr/bin/env bash
# ------------------------------------------------------------------------------
# Installation script for the ICON model.
# ------------------------------------------------------------------------------

# Set constants.
ICON_BRANCH="release-2024.7-public"
SCRIPT_DIR=$(dirname "$(realpath "${BASH_SOURCE[0]}")")
PREFIX="${PREFIX:=$SCRIPT_DIR}"
SOURCE_DIR="$PREFIX/icon-model"

# Load dependencies.
source "$SCRIPT_DIR/helpers.sh"

# Print usage of this script.
print_usage() {
    # Print usage by default.
    echo "========="
    echo "| USAGE |"
    echo "========="
    echo -e "By default all installations are done out-of-source to allow for " \
        "multiple versions."
    echo -e "Version are passed via 3 variables: the config directory, the " \
        "config file name, and the verion name, all seperated by ':' " \
        "characters."
    echo "For example, installing the CPU version on Levante could be:"
    echo -e "\t./install.sh dkrz:levante.cpu.nvhpc-23.9:cpu"
    echo "Multiple version can be specified for chained installations, e.g.:"
    echo -e "\t./install.sh dkrz:levante.cpu.nvhpc-23.9:cpu " \
        "dkrz:levante.gpu.nvhpc-23.9:gpu"
    echo -e "\n"
    echo "In order to load default modules of a machine, set the MACHINE " \
        "environment variable, e.g.:"
    echo -e "\t export MACHINE=levante"
    echo "Or set it temporarily while running the installation script, e.g.:"
    echo -e "\t MACHINE=levante ./install.sh dkrz:levante.cpu.nvhpc-23.9:cpu"
    echo -e "\n"
    echo "Some environment variables that can be set for altering behavior:"
    echo -e "\tICON_BRANCH\tSet the ICON branch to use as source. Defaults " \
        "to $ICON_BRANCH"
    echo -e "\tPREFIX\tSet the directory in which to clone and install icon. " \
        "Defaults to location of this script."
    echo -e "\n\n"
}

# Load machine-specific modules.
load_modules() {
    # Load modules depending on the set MACHINE.
    case "$MACHINE" in
        "levante")
            echo "Loading Levante modules.."
            module load git
            ;;
        *)
            echo "Env variable $$MACHINE not set, not loading modules."
    esac
}

# Install a provided version of ICON out-of-source.
install() {
    # Check if the required variables are set.
    CONF_DIR = $1
    SCRIPT = $2
    VERSION = $3

    if [[ -z "$CONF_DIR" || -z "$SCRIPT" ]]; then
        info "[INSTALL]\tMissing $$CONF_DIR, $$SCRIPT, or $$VERSION, skipping.."
        return
    fi
    
    # Create a build directory for this version.
    VERSION_DIR="$SOURCE_DIR/build/$VERSION"
    mkdir -p "$VERSION_DIR"
    cd "$VERSION_DIR"

    # Configure.
    $SOURCE_DIR/config/$CONF_DIR/$SCRIPT --enable-openmp

    # Build
    make -j8

    # Migrate runscripts and data to this version.
    rsync -uavz "$SOURCE_DIR/run" . --exclude='*in' --exclude='.*'
    rsync -uavz "$SOURCE_DIR/externals" . --exclude='.git' \
        --exclude='*.f90' --exclude='*.F90' --exclude='*.c' --exclude='*.h' \
        --exclude='*.Po' --exclude='tests' --exclude='rrtmgp*.nc' \
        --exclude='*.mod' --exclude='*.o'
    rsync -uavz $SOURCE_DIR/make_runscripts .
    ln -sf "$SOURCE_DIR/data"
    ln -sf "$SOURCE_DIR/vertical_coord_tables"

    # Create all runscripts.
    $SOURCE_DIR/make_runscripts --all
    success "Succesfully installed $CONF_DIR/$SCRIPT"
}

# Main function.
main() {
    # Print usage by default.
    print_usage
   
    # Load machine specific modules.
    load_modules

    # Check whether the source files are present, download if missing.
    if [[ ! -d "$SOURCE_DIR" ]]; then
        echo -e "Source is missing, downloading.."
        
        # Exit if git is unavailable
        test_command "git"

        # Clone source.
        git clone --branch "$ICON_BRANCH" --single-branch \
            https://gitlab.dkrz.de/icon/icon-model.git $SOURCE_DIR
    fi

    # Parse version to install.
    for ARG in "$@"; do
        IFS=':' read -r CONF_DIR SCRIPT VERSION <<< "$ARG"
    
        # Check if both variables are set.
        if [[ -z "$CONF_DIR" || -z "$SCRIPT" || -z "$VERSION" ]]; then
            fatal "Invalid input format: $ARG, expected dir:filename:version"
            exit 1
        fi

        # Check if the configuration directory exists.
        if [[ ! -d "$SOURCE_DIR/config/$CONF_DIR" ]]; then
            info "[INSTALL]\t$$CONF_DIR '$CONF_DIR' does not exist, skipping"
            continue
        fi

        # Check if the configuration script exists.
        if [[ ! -f "$SOURCE_DIR/confg/$CONF_DIR/$SCRIPT" ]]; then
            info "[INSTALL]\t$$SCRIPT '$SCRIPT' does not exist within $CONF_DIR," \
                "skipping"
            continue
        fi  

        echo "Installing $CONF_DIR/$SCRIPT"
        install $CONF_DIR $SCRIPT
    done
} 

# Execute main.
main "$@"
